@@ -6,7 +6,7 @@ use strict;
 use vars qw( $VERSION @ISA );
 
 BEGIN {
-    $VERSION = '1.18';
+    $VERSION = '1.26';
     @ISA     = qw( Archive::Zip );
 }
 
@@ -130,8 +130,9 @@ sub desiredCompressionMethod {
         $self->{'desiredCompressionMethod'} = $newDesiredCompressionMethod;
         if ( $newDesiredCompressionMethod == COMPRESSION_STORED ) {
             $self->{'desiredCompressionLevel'} = 0;
-        }
-        elsif ( $oldDesiredCompressionMethod == COMPRESSION_STORED ) {
+            $self->{'bitFlag'} &= ~GPBF_HAS_DATA_DESCRIPTOR_MASK;
+
+        } elsif ( $oldDesiredCompressionMethod == COMPRESSION_STORED ) {
             $self->{'desiredCompressionLevel'} = COMPRESSION_LEVEL_DEFAULT;
         }
     }
@@ -189,12 +190,20 @@ sub externalFileAttributes {
 }
 
 # Convert UNIX permissions into proper value for zip file
-# NOT A METHOD!
+# Usable as a function or a method
 sub _mapPermissionsFromUnix {
-    my $perms = shift;
-    return $perms << 16;
+    my $self    = shift;
+    my $mode    = shift;
+    my $attribs = $mode << 16;
 
-    # TODO: map MS-DOS perms too (RHSA?)
+    # Microsoft Windows Explorer needs this bit set for directories
+    if ( $mode & DIRECTORY_ATTRIB ) {
+        $attribs |= 16;
+    }
+
+    return $attribs;
+
+    # TODO: map more MS-DOS perms
 }
 
 # Convert ZIP permissions into Unix ones
@@ -284,18 +293,17 @@ sub _mapPermissionsToUnix {
 
 sub unixFileAttributes {
     my $self     = shift;
-    my $oldPerms = $self->_mapPermissionsToUnix();
-    if (@_) {
+    my $oldPerms = $self->_mapPermissionsToUnix;
+    if ( @_ ) {
         my $perms = shift;
-        if ( $self->isDirectory() ) {
+        if ( $self->isDirectory ) {
             $perms &= ~FILE_ATTRIB;
             $perms |= DIRECTORY_ATTRIB;
-        }
-        else {
+        } else {
             $perms &= ~DIRECTORY_ATTRIB;
             $perms |= FILE_ATTRIB;
         }
-        $self->{'externalFileAttributes'} = _mapPermissionsFromUnix($perms);
+        $self->{externalFileAttributes} = $self->_mapPermissionsFromUnix($perms);
     }
     return $oldPerms;
 }
@@ -388,6 +396,8 @@ sub extractToFileNamed {
     return _ioError("Can't open file $name for write") unless $status;
     my $retval = $self->extractToFileHandle($fh);
     $fh->close();
+    chmod ($self->unixFileAttributes(), $name)
+        or return _error("Can't chmod() ${name}: $!");
     utime( $self->lastModTime(), $self->lastModTime(), $name );
     return $retval;
 }
@@ -511,7 +521,7 @@ sub _writeLocalFileHeader {
     my $fh   = shift;
 
     my $signatureData = pack( SIGNATURE_FORMAT, LOCAL_FILE_HEADER_SIGNATURE );
-    $fh->print($signatureData)
+    $self->_print($fh, $signatureData)
       or return _ioError("writing local header signature");
 
     my $header = pack(
@@ -527,13 +537,13 @@ sub _writeLocalFileHeader {
         length( $self->localExtraField() )
     );
 
-    $fh->print($header) or return _ioError("writing local header");
+    $self->_print($fh, $header) or return _ioError("writing local header");
     if ( $self->fileName() ) {
-        $fh->print( $self->fileName() )
+        $self->_print($fh, $self->fileName() )
           or return _ioError("writing local header filename");
     }
     if ( $self->localExtraField() ) {
-        $fh->print( $self->localExtraField() )
+        $self->_print($fh, $self->localExtraField() )
           or return _ioError("writing local extra field");
     }
 
@@ -546,7 +556,7 @@ sub _writeCentralDirectoryFileHeader {
 
     my $sigData =
       pack( SIGNATURE_FORMAT, CENTRAL_DIRECTORY_FILE_HEADER_SIGNATURE );
-    $fh->print($sigData)
+    $self->_print($fh, $sigData)
       or return _ioError("writing central directory header signature");
 
     my $fileNameLength    = length( $self->fileName() );
@@ -573,18 +583,18 @@ sub _writeCentralDirectoryFileHeader {
         $self->writeLocalHeaderRelativeOffset()
     );
 
-    $fh->print($header)
+    $self->_print($fh, $header)
       or return _ioError("writing central directory header");
     if ($fileNameLength) {
-        $fh->print( $self->fileName() )
+        $self->_print($fh,  $self->fileName() )
           or return _ioError("writing central directory header signature");
     }
     if ($extraFieldLength) {
-        $fh->print( $self->cdExtraField() )
+        $self->_print($fh,  $self->cdExtraField() )
           or return _ioError("writing central directory extra field");
     }
     if ($fileCommentLength) {
-        $fh->print( $self->fileComment() )
+        $self->_print($fh,  $self->fileComment() )
           or return _ioError("writing central directory file comment");
     }
 
@@ -607,7 +617,7 @@ sub _writeDataDescriptor {
         $self->uncompressedSize()
     );
 
-    $fh->print($header)
+    $self->_print($fh, $header)
       or return _ioError("writing data descriptor");
     return AZ_OK;
 }
@@ -637,7 +647,7 @@ sub _refreshLocalFileHeader {
         length( $self->localExtraField() )
     );
 
-    $fh->print($header)
+    $self->_print($fh, $header)
       or return _ioError("re-writing local header");
     $fh->seek( $here, IO::Seekable::SEEK_SET )
       or return _ioError("seeking after rewrite of local header");
@@ -932,7 +942,7 @@ sub _writeData {
         return $status if ( $status != AZ_OK and $status != AZ_STREAM_END );
 
         if ( length($$outRef) > 0 ) {
-            $writeFh->print($$outRef)
+            $self->_print($writeFh, $$outRef)
               or return _ioError("write error during copy");
         }
 
